from __future__ import absolute_import

cli = None

def init_cli():
    import click
    from . import config

    @click.group("catalyze")
    @click.option("--baas-host", help = "Alternate BaaS API URL")
    @click.option("--paas-host", help = "Alternate PaaS API URL")
    @click.option("--skip-validation", is_flag = True, help = "Skip certificate validation")
    def inner_cli(baas_host, paas_host, skip_validation):
        if baas_host is not None:
            config.baas_host = baas_host
            print("Overriding BaaS URL: " + config.baas_host)
        if paas_host is not None:
            config.paas_host = paas_host
            print("Overriding PaaS URL: " + config.paas_host)
        if skip_validation:
            config.behavior["skip_cert_validation"] = False
            print("Skipping cert validation, I hope this was intentional")

    global cli
    cli = inner_cli

    @cli.command()
    def version():
        """Outputs the current CLI version."""
        print(config.version)

    from catalyze.commands import \
        associate, dashboard, environments, rake, redeploy, status, variables

def run():
    import catalyze.__main__



from __future__ import absolute_import

import click
from catalyze import cli, client, git, project
from catalyze.helpers import environments, services

@cli.command(short_help = "Associates a local repository with an environment.")
@click.argument("env_label")
@click.option("--remote", help = "The git remote to be created", default = "catalyze")
def associate(env_label, remote):
    """Associates the git repository in the current directory. This caches the session token and the environment ID, and creates a git remote."""
    session = client.acquire_session()
    for env in environments.list(session):
        if env["data"]["name"] == env_label:
            settings = {
                    "token": session.token,
                    "environmentId": env["environmentId"]
                }
            for svc in services.list(session, env["environmentId"]):
                if svc["data"]["type"] == "code":
                    if remote in git.remote_list():
                        git.remote_remove(remote)
                    git.remote_add(remote, svc["data"]["source"])
                    settings["serviceId"] = svc["serviceId"]
                    project.save_settings(settings)
                    print("\"%s\" remote added." % (remote,))
                    return
            print("No code service found for \"%s\" environment (%s)" % (env_label, env["environmentId"]))
            return
    print("No environment with label \"%s\" found." % (env_label,))

@cli.command()
def disassociate():
    """Remove association with environment."""
    project.clear_settings()
    print("Association cleared.")



from __future__ import absolute_import

import json
from catalyze import cli, client, project
from catalyze.helpers import environments, services

@cli.command()
def status():
    """Check the status of the environment and service."""
    settings = project.read_settings()
    session = client.acquire_session(settings)
    env = environments.retrieve(session, settings["environmentId"])
    print("environment state: " + env["state"])
    codes = []
    noncodes = []
    for service in services.list(session, settings["environmentId"]):
        if service["data"]["type"] is not None:
            if service["data"]["type"] == "code":
                codes.append("\t%s (size = %s, build status = %s, deploy status = %s)" % (service["data"]["label"], service["data"]["size"], service["data"]["build_status"], service["data"]["deploy_status"]))
            else:
                noncodes.append("\t%s (size = %s, image = %s, status = %s)" % (service["data"]["label"], service["data"]["size"], service["data"]["name"], service["data"]["deploy_status"]))
        elif service["data"]["name"] == "service_proxy":
            pass
    for item in (codes + noncodes):
        print(item)



from __future__ import absolute_import

from catalyze import config

def list(session, env_id, svc_id):
    route = "%s/v1/environments/%s/services/%s/jobs" % (config.paas_host, env_id, svc_id)
    return session.get(route, verify = True)

def retrieve(session, env_id, svc_id, job_id):
    route = "%s/v1/environments/%s/services/%s/jobs/%s" % (config.paas_host, env_id, svc_id, job_id)
    return session.get(route, verify = True)



from __future__ import absolute_import

from catalyze import config, project
import requests, json, getpass, os

class AuthError(Exception):
    def __init__(self, message):
        Exception.__init__(self, message)

class ClientError(Exception):
    def __init__(self, resp):
        Exception.__init__(self, "%d: %s" % (resp.status_code, resp.text))

def is_ok(resp):
    return resp.status_code >= 200 and resp.status_code < 300

class Session:
    def __init__(self, token = None, user_id = None, username = None, password = None):
        self.session = requests.Session()
        self.session.verify = "skip_cert_validation" not in config.behavior
        if token is None:
            self.sign_in(username, password)
        else:
            self.token = token
            self.user_id = user_id

    def _build_headers(self):
        return {
            "X-Api-Key": config.api_key,
            "Accept": "application/json",
            "Content-Type": "application/json",
            "Authorization": "Bearer " + self.token,
            "X-CLI-Version": config.version
        }

    def get(self, url, verify = False):
        resp = self.session.get(url, headers = self._build_headers())
        if verify:
            if is_ok(resp):
                return None if not resp.text else resp.json()
            else:
                raise ClientError(resp)
        else:
            return resp

    def post(self, url, body, verify = False):
        resp = self.session.post(url, headers = self._build_headers(), data = json.dumps(body))
        if verify:
            if is_ok(resp):
                return None if not resp.text else resp.json()
            else:
                raise ClientError(resp)
        else:
            return resp

    def post_file(self, url, form, verify = False):
        headers = self._build_headers()
        del headers["Content-Type"]
        resp = self.session.post(url, headers = headers, files = form)
        if verify:
            if is_ok(resp):
                return None if not resp.text else resp.json()
            else:
                raise ClientError(resp)
        else:
            return resp

    def put(self, url, body, verify = False):
        resp = self.session.put(url, headers = self._build_headers(), data = json.dumps(body))
        if verify:
            if is_ok(resp):
                return None if not resp.text else resp.json()
            else:
                raise ClientError(resp)
        else:
            return resp

    def delete(self, url, verify = False):
        resp = self.session.delete(url, headers = self._build_headers())
        if verify:
            if is_ok(resp):
                return None if not resp.text else resp.json()
            else:
                raise ClientError(resp)
        else:
            return resp

    def sign_in(self, username, password):
        resp = self.session.post(config.baas_host + "/v2/auth/signin", headers = {
                "X-Api-Key": config.api_key,
                "Accept": "application/json",
                "Content-Type": "application/json"
            }, data = json.dumps({
                "username": username,
                "password": password
            }))
        if is_ok(resp):
            j = resp.json()
            self.token = j["sessionToken"]
            self.user_id = j["usersId"]
        else:
            raise AuthError(resp.json())

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.session.close()

def acquire_session(settings = None):
    if settings is not None and "token" in settings and "user_id" in settings:
        session = Session(token = settings["token"], user_id = settings["user_id"])
        resp = session.get(config.baas_host + "/v2/auth/verify")
        if resp.status_code == 200:
            return session
        elif resp.status_code == 401:
            print("Session has timed out. Please re-enter credentials.")
    username = os.getenv("CATALYZE_USERNAME")
    if username is None:
        username = raw_input("Username: ") if "username" not in config.behavior else config.behavior["username"]
    password = os.getenv("CATALYZE_PASSWORD")
    if password is None:
        password = getpass.getpass("Password: ")
    session = Session(username = username, password = password)
    if settings is not None:
        settings["token"] = session.token
        settings["user_id"] = session.user_id
        project.save_settings(settings)
    return session



from __future__ import absolute_import

from catalyze import cli, client, config, project
from catalyze.helpers import environments

@cli.command("environments")
def list_environments():
    """Lists all environments you own."""
    settings = project.read_settings(required = False)
    session = client.acquire_session(settings)
    envs = environments.list(session)
    for env in envs:
        print("%s: %s (state: %s)" % (env["data"]["name"], env["environmentId"], env["state"]))
    if len(envs) == 0:
        print("no environments found")
